﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace MyECULib
{
    public class MyECUDataAnalogInputs
    {
        public string source;
        public double WBO2Sensor0;
        public double OilTemp;
        public double AirTemp;
        public double AirPsensorVoltage;
        public uint Throttle;
        public double BatteryVolts;
        public double WBO2Sensor1;
                
        private static Regex reg = new Regex(@"^([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})$");

        override public string ToString()
        {
            return "SensorInfo " + source + "\n" +
            "O2: " + System.Math.Round(WBO2Sensor0, 2) + " V\n" +
            "OilT: " + System.Math.Round(OilTemp, 1) + " °C\n" +
            "AirT: " + System.Math.Round(AirTemp, 1) + " °C\n" +
            "AirP: " + System.Math.Round(AirPsensorVoltage, 2) + " V\n" +
            "Thr: " + Throttle + "\n" +
            "Bat: " + System.Math.Round(BatteryVolts, 1) + " V\n\n";
        }

        public MyECUDataAnalogInputs(String line, String data)
        {
            source = line;
            Match m = reg.Match(data);
            if (m.Success)
            {
                WBO2Sensor0 = MyECUData.hexToVolt(m.Groups[1].ToString(), MyECUData.voltages.Volt5);
                OilTemp = MyECUData.hexToTemp(m.Groups[2].ToString());
                AirTemp = MyECUData.hexToTemp(m.Groups[3].ToString());
                AirPsensorVoltage = MyECUData.hexToVolt(m.Groups[4].ToString(), MyECUData.voltages.Volt5);
                Throttle = MyECUData.hexToUInt(m.Groups[5].ToString());
                BatteryVolts = MyECUData.hexToVolt(m.Groups[6].ToString(), MyECUData.voltages.Volt12);
                WBO2Sensor1 = MyECUData.hexToVolt(m.Groups[7].ToString(), MyECUData.voltages.Volt5);
            }
            else
            {
                //throw new Exception("Bad data");
            }
        }
    }
}
/*
Analog inputs
=============
This structure provides the raw analog signals as seen be the ECU
This structure is prefixed with an "=0".

		typedef struct {
		    uint16_t    GasSensor;
		    uint16_t    OilTemp;
		    uint16_t    AirTemp;
		    uint16_t    AirPressure;
		    uint16_t    Throttle;
		    uint16_t    BatteryVolts;
		    uint16_t    2ndGasSensor;
		} Analogs_t;

Map (7.4,11.2) RPM=3071 Inj=12175us BaroAdj=106.7% o2Adj=100.0% o2[1] o2Target=0.00V o2=1.85V OilT=67.0C AirT=9.0C AirP=1020mb Thro=731 Bat=13.92V

=0 0E 7A01 1F01 3403 A903 DB02  F401 9000 E0   hex LE
       378  287  820  937  731   500  144      dec
      1.85 2.40 4.00 4.58 3.57  2.44 0.70      volt
                               13.92           volt * 5.70
*/