﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace MyECULib
{
    public class MyECUDataDiagnosticInfo
    {
        public string source;
        public uint Alert;  // 1 bit
        public uint res;    // 3 bits
        public uint BadPulse; // 2 bits
        public uint BadSyncs; // 2 bits

        public uint MissedPulse; // 2 bits
        public uint InvSync; // 2 bits
        public uint Noise; // 2 bits
        public uint Noise1; // 2 bits

        public uint InvSpk0; // 2 bits
        public uint InvSpk1; // 2 bits
        public uint InvInj0; // 2 bits
        public uint InvInj1; // 2 bits

        public uint xSpk0; // 4 bits
        public uint xSpk1; // 4 bits
       
        public uint xInj0; // 4 bits
        public uint xInj1; // 4 bits

        public uint p0;
        public uint p1;
        public uint p2;

        private static Regex reg = new Regex(@"([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F])([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})$");

        private static void unpackOverloadedInfo(uint data, ref double adjust, ref uint index, ref uint active)
        {
            adjust = (data & 2047);
            adjust = ((adjust/1024)*100);
            index = (data >> 12) & 7;
            active = (data >> 15);
        }

        override public string ToString()
        {
            return "DiagnosticInfo " + source + "\n" +
            "Alert: " + Alert + "\n" +
            "res: " + res + "\n" +
            "BadPulse: " + BadPulse + "\n" +
            "BadSyncs: " + BadSyncs + "\n" +
            "MissedPulse: " + MissedPulse + "\n" +
            "InvSync: " + InvSync + "\n" +
            "Noise: " + Noise + "\n" +
            "Noise1: " + Noise1 + "\n" +
            "InvSpk0: " + InvSpk0 + "\n" +
            "InvSpk1: " + InvSpk1 + "\n" +
            "InvInj0: " + InvInj0 + "\n" +
            "InvInj1: " + InvInj1 + "\n" +
            "xSpk0: " + xSpk0 + "\n" +
            "xSpk1: " + xSpk1 + "\n" +
            "xInj0: " + xInj0 + "\n" +
            "xInj1: " + xInj1 + "\n" +
            "p0: " + p0 + "\n" +
            "p1: " + p1 + "\n" +
            "p2: " + p2 + "\n\n";
        }

        public MyECUDataDiagnosticInfo(String line, String data)
        {
            source = line;
            Match m = reg.Match(data);
            if (m.Success)
            {
                Alert = uint.Parse(m.Groups[2].ToString(), System.Globalization.NumberStyles.HexNumber) & 1;
                res = uint.Parse(m.Groups[2].ToString(), System.Globalization.NumberStyles.HexNumber) >> 1;
                BadPulse = uint.Parse(m.Groups[1].ToString(), System.Globalization.NumberStyles.HexNumber) & 3;
                BadSyncs = uint.Parse(m.Groups[1].ToString(), System.Globalization.NumberStyles.HexNumber) >> 2;
                MissedPulse = uint.Parse(m.Groups[4].ToString(), System.Globalization.NumberStyles.HexNumber) & 3;
                InvSync = uint.Parse(m.Groups[4].ToString(), System.Globalization.NumberStyles.HexNumber) >> 2;
                Noise = uint.Parse(m.Groups[3].ToString(), System.Globalization.NumberStyles.HexNumber) & 3;
                Noise1 = uint.Parse(m.Groups[3].ToString(), System.Globalization.NumberStyles.HexNumber) >> 2;
                InvSpk0 = uint.Parse(m.Groups[6].ToString(), System.Globalization.NumberStyles.HexNumber) & 3;
                InvSpk1 = uint.Parse(m.Groups[6].ToString(), System.Globalization.NumberStyles.HexNumber) >> 2;
                InvInj0 = uint.Parse(m.Groups[5].ToString(), System.Globalization.NumberStyles.HexNumber) & 3;
                InvInj1 = uint.Parse(m.Groups[5].ToString(), System.Globalization.NumberStyles.HexNumber) >> 2;
                xSpk0 = uint.Parse(m.Groups[8].ToString(), System.Globalization.NumberStyles.HexNumber);
                xSpk1 = uint.Parse(m.Groups[7].ToString(), System.Globalization.NumberStyles.HexNumber);
                xInj0 = uint.Parse(m.Groups[10].ToString(), System.Globalization.NumberStyles.HexNumber);
                xInj1 = uint.Parse(m.Groups[9].ToString(), System.Globalization.NumberStyles.HexNumber);
                p0 = MyECUData.hexToUInt(m.Groups[11].ToString());
                p1 = MyECUData.hexToUInt(m.Groups[12].ToString());
                p2 = MyECUData.hexToUInt(m.Groups[13].ToString());
            }
            else
            {
                //throw new Exception("Bad data");
            }
        }
    }
}
/*
They are diagnostic outputs. Like an assert statement firing.

You might see a few of these at startup but you shouldn't really see them in normal running.

This is the structure. I'll leave it to you to see if you can figure out which fields are changing.

typedef struct
{
    uint8_t Alert:1;
    uint8_t res:3;
    uint8_t BadPulse:2;
    uint8_t BadSyncs:2;

    uint8_t MissedPulse:2;
    uint8_t InvSync:2;
    uint8_t Noise:2;
    uint8_t Noise1:2;

    uint8_t InvSpk0:2;
    uint8_t InvSpk1:2;
    uint8_t InvInj0:2;
    uint8_t InvInj1:2;

    uint8_t xSpk0:4;
    uint8_t xSpk1:4;
   
    uint8_t xInj0:4;
    uint8_t xInj1:4;

    uint16_t p0;
    uint16_t p1;
    uint16_t p2;
    } Diag_t;

*/