﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace MyECULib
{
     
    public class MyECUDataTimingInfoFullPulse
    {
        public uint TimeWait; // Time in us after the pulse
        public Byte Pulse;  // The pulse number - 0..47 for My16M

        public MyECUDataTimingInfoFullPulse(String _timeWait,String _Pulse)
        {
            TimeWait = MyECUData.hexToUInt(_timeWait);
            Pulse = Byte.Parse(_Pulse, System.Globalization.NumberStyles.HexNumber);
        }

        public double fullPulseToBTDC(uint PulsePeriod)
        {
            double res = (MyECUSource.TDCPulse - (Pulse + ((1.0 * TimeWait) / PulsePeriod))) * MyECUSource.DegPerPulse;
            if (res < 0)
            {
                res += 720;
            }
            return res;
        }

        public override string ToString()
        {
            return ToPulseString() + " plus TimeWait of " + TimeWait + " us";
        }

        public string ToPulseString()
        {
            return "Pulse " + Pulse;
        }

    }

    public class MyECUDataTimingInfo
    {
        public string source;
	    public MyECUDataTimingInfoFullPulse SparkOff; // Position of spark pulse trailing edge - the spark
		public MyECUDataTimingInfoFullPulse SparkOn; // Position of spark pulse leading edge - coil charging
        public uint PulsePeriod; // The time(us) between engine timing pulses
        public uint InjDuration0; // Pulse duration in us (incl. BaroAdj and O2Adj, excl. Deadtime)
        public uint InjDuration1; // Same for cylinder 1
		public MyECUDataTimingInfoFullPulse InjAdvance; // Position of injector (End of Injection)
        public byte MapIdx; // The index of the map entry being used ( lower left )
        public byte MapIdy; // map Id y
        public byte fy; // The fraction throttle from base map entry
        public byte fx; // The fraction RPM from base map entry
        public double DegPerPulse = (720.0 / MyECUSource.PulsesPer); // Degrees per pulse from timing wheel

        private static Regex reg = new Regex(@"^([0-9A-F]{4})([0-9A-F]{2})([0-9A-F]{4})([0-9A-F]{2})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{4})([0-9A-F]{2})([0-9A-F])([0-9A-F])([0-9A-F]{2})([0-9A-F]{2})$");
        //                                           1            2              3          4           5             6             7          8            9            10        11        12            13

        public string ToString(uint DeadTime)
        {
            return "TimingInfo "+source+"\n" +
            "RPM: "+RPM()+"\n" +
            "PulsePeriod: " + PulsePeriod + " us between pulses (" + PulsePeriod / (720 / MyECUSource.PulsesPer) + " us per degree of rotation)\n" +
            "Injduration0: " + InjDuration0 + " us (" + System.Math.Round(InjStart0(DeadTime) - InjEnd(), 1) + "°), " + "Duty Cycle: " + System.Math.Round(DutyCycle0(), 1) + "%\n" +
            "Injduration1: " + InjDuration1 + " us (" + System.Math.Round(InjStart1(DeadTime) - InjEnd(), 1) + "°), " + "Duty Cycle: " + System.Math.Round(DutyCycle1(), 1) + "%\n" +
            "SoI0: " + /* InjAdvance.ToPulseString() + " plus InjDur of " + InjDuration0 + " us plus InjDeadTime of " + DeadTime + " us  =>  " + */ System.Math.Round(InjStart0(DeadTime), 1) + " °BTDC\n" +
            "SoI1: " + /* InjAdvance.ToPulseString() + " plus InjDur of " + InjDuration1 + " us plus InjDeadTime of " + DeadTime + " us  =>  " + */ System.Math.Round(InjStart1(DeadTime), 1) + " °BTDC\n" +
            "EoI: " + /* InjAdvance.ToPulseString() + "  =>  " + */ System.Math.Round(InjEnd(), 1) + " °BTDC\n" +
            "SparkOn: " + /* SparkOn.ToString() + "   =>   " + */ System.Math.Round(SpkChargeStart(), 1) + " °BTDC\n" +
            "SparkOff: " + /* SparkOff.ToString() + "   =>   " + */ System.Math.Round(SkpAdv(),1) + " °BTDC\n" +
            "Map: (" + System.Math.Round(MapThr_x(),1) + " , " + System.Math.Round(MapRpm_y(),1) + ")\n\n";
        }

        public double DutyCycle0()  // 0-100 %
        {
            return (100.0 * InjDuration0 / (PulsePeriod * MyECUSource.PulsesPer));
        }

        public double DutyCycle1()  // 0-100 %
        {
            return (100.0 * InjDuration1 / (PulsePeriod * MyECUSource.PulsesPer));
        }

        public uint RPM()
        {
            // Without the cludge we would return 200 instead of zero, just like the tacho
            // because a PulsePeriod of 12499 is reported from ECU
            if (PulsePeriod == 12499)
            {
                return 0;
            }
            return 60000000 / (PulsePeriod * (MyECUSource.PulsesPer / 2));
        }

        public double SpkChargeStart()
        {
            return SparkOn.fullPulseToBTDC(PulsePeriod);
        }

        public double SkpAdv()
        {
            return SparkOff.fullPulseToBTDC(PulsePeriod);
        }

        public double InjEnd()
        {
            return InjAdvance.fullPulseToBTDC(PulsePeriod);
        }

        public double usToDegrees(uint us)
        {
            return (((double)us / (double)PulsePeriod) * DegPerPulse);
        }

        public double InjStart0(uint DeadTime)
        {
            if (InjDuration0 == 0)
            {
                return InjAdvance.fullPulseToBTDC(PulsePeriod);
            }
            else
            {
                return InjAdvance.fullPulseToBTDC(PulsePeriod) + usToDegrees(InjDuration0 + DeadTime);
            }
        }

        public double InjStart1(uint DeadTime)
        {
            if (InjDuration1 == 0)
            {
                return InjAdvance.fullPulseToBTDC(PulsePeriod);
            }
            else
            {
                return InjAdvance.fullPulseToBTDC(PulsePeriod) + usToDegrees(InjDuration1 + DeadTime);
            }
        }

        public double MapThr_x()
        {
            return MapIdx + (fx / 256.0);
        }

        public double MapRpm_y()
        {
            return MapIdy + (fy / 256.0);
        }

        public MyECUDataTimingInfo(String line, String data)
        {
            source = line;
            Match m = reg.Match(data);
            if (m.Success)
            {
                // SparkOff
                string spoffW = m.Groups[1].ToString();
                string spoffP = m.Groups[2].ToString();
                SparkOff = new MyECUDataTimingInfoFullPulse(spoffW, spoffP);

                // SparkOn
                string sponW = m.Groups[3].ToString();
                string sponP = m.Groups[4].ToString();
                SparkOn = new MyECUDataTimingInfoFullPulse(sponW, sponP);
   
                PulsePeriod = MyECUData.hexToUInt(m.Groups[5].ToString());     
                InjDuration0 = MyECUData.hexToUInt(m.Groups[6].ToString());
                InjDuration1 = MyECUData.hexToUInt(m.Groups[7].ToString());

                // InjAdvance
                string injadvW = m.Groups[8].ToString();
                string injadvP = m.Groups[9].ToString();
                InjAdvance = new MyECUDataTimingInfoFullPulse(injadvW, injadvP);

                // Map coordinate
                string t = m.Groups[10].ToString();
                MapIdy = Byte.Parse(t, System.Globalization.NumberStyles.HexNumber);
                string r = m.Groups[11].ToString();
                MapIdx = Byte.Parse(r, System.Globalization.NumberStyles.HexNumber);
                string tf = m.Groups[12].ToString();
                fy = Byte.Parse(tf, System.Globalization.NumberStyles.HexNumber);
                string rf = m.Groups[13].ToString();
                fx = Byte.Parse(rf, System.Globalization.NumberStyles.HexNumber);
            }
            else
            {
                //throw new Exception("Bad data");
            }
        }
    }
}
/*
Timing Info
===========
This structure can be used to determine the engine speed, throttle setting and postion within the map, and the resultant 
engine pulses
This structure is prefixed with a "=1".

		typedef struct {
				FullPulse_t SparkOff; // Position of spark pulse trailing edge - the spark
				FullPulse_t SparkOn; // Position of spark pulse leading edge - coil charging
				uint16_t PulsePeriod; // The time(us) between engine timing pulses
				uint16_t InjDuration0; // Pulse duration in us
				uint16_t InjDuration1; // Pulse duration in us
				FullPulse_t InjAdvance; // Position of injector
				uint8_t MapIdx_; // The index of the map entry being used ( lower left )                                 
				uint8_t fy; // The fraction throttle from base map entry
				uint8_t fx; // The fraction RPM from base map entry
		} TimingInfo_t;

FullPulse_t has the form

		typedef struct {
		    uint16_t    TimeWait;        // Time in us after the pulse
		    uint8_t     Pulse;            // The pulse number - 0..47 for My16M
		}   FullPulse_t;
 
Map (7.4,11.2) RPM=3071 Inj=12175us BaroAdj=106.7% o2Adj=100.0% o2[1] o2Target=0.00V o2=1.85V OilT=67.0C AirT=9.0C AirP=1020mb Thro=731 Bat=13.92V
=1 12 7F00 14 D402 10 2E03  8F2F  8F2F 0000 23   B 7 3F  68 A7	hex LE
       127 20  724 16  814 12175 12175    0 35  11,7 63 104			dec
      p20+127 p16+724                    p35+0   T,R  t   r
60000000/(814*24) = 3071 rpm (60'' us per minute, 24 pulses per crank rotation)

Map (7.8,4.1) RPM=3242 Inj=5977us BaroAdj=106.0% o2Adj=102.2% o2[3] o2Target=2.39V o2=2.09V OilT=80.0C AirT=11.0C AirP=1020mb Thro=372 Bat=13.92V
=1 12 0302 13 9200 10 0303 5917 5917 0000 23 4 7 0D  C5 DC
       515 19  146 16  771 5977 5977    0 35 4,7 13 197
60000000/(771*24) = 3242 rpm
 
*/
