﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;

namespace MyECULib
{
    public class MyECUSerial : MyECUSource
    {
        private string portName = null;
        private SerialPort serialPort = null;
        private int timeouts = 0;

        public MyECUSerial(string _portName)
        {
            portName = _portName;
            serialPort = new SerialPort(portName);
            serialPort.BaudRate = 57600;
            serialPort.DataBits = 8;
            serialPort.Parity = System.IO.Ports.Parity.None;
            serialPort.StopBits = System.IO.Ports.StopBits.One;
            serialPort.Handshake = System.IO.Ports.Handshake.None;
            serialPort.NewLine = "\r";
            serialPort.ReadTimeout = 1000;
        }

        public static String[] getPortNames()
        {
            return SerialPort.GetPortNames();
        }

        override public bool isOpen()
        {
            if (serialPort != null)
            {
                return serialPort.IsOpen;
            }
            else
            {
                return false;
            }
        }

        override public bool Open()
        {
            try
            {
                serialPort.Open();
                GC.SuppressFinalize(serialPort.BaseStream);
                return serialPort.IsOpen;
            }
            catch
            {
                return false;
            }

        }

        override public void Close()
        {
            if (serialPort != null)
            {
                try
                {
                    serialPort.Close();
                    serialPort = null;
                }
                catch
                {
                    Console.Error.WriteLine("Error closing serial port!?");
                }
                finally
                {
                    throwAwaySerialPort();
                }
            }
        }

        override public int Timeouts()
        {
            return timeouts;
        }

        public void throwAwaySerialPort()
        {
            try
            {
                serialPort = null;
            }
            catch
            {
                Console.Error.WriteLine("doh! Could not discard serialPort object!?");
            }
        }
 
        override public void DiscardBuffer()
        {
            timeouts = 0;
            serialPort.DiscardInBuffer();
            serialPort.ReadExisting();
        }

        override public string getData()
        {
            try
            {
                return serialPort.ReadLine();
            }
            catch (TimeoutException)
            {
                timeouts++;
                Thread.Sleep(15);  // workaround for br0ken devices that does not block on readline()
                return "";
            }
            catch (InvalidOperationException)
            {
                Close();
                return "";
            }
            catch (System.IO.IOException)
            {
                Close();
                return "";
            }
        }

    }
}
