﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace MyECULib
{
    abstract public class MyECUSource
    {
        public static uint PulsesPer = MyECULib.Properties.Settings.Default.PulsesPer; // 48;
        public static uint CylOffs = MyECULib.Properties.Settings.Default.CylOffs; // 270;
        public static uint TDCPulse = MyECULib.Properties.Settings.Default.TDCpulse; // 0x16;
        public static double LC1zero = MyECULib.Properties.Settings.Default.LC1zero; // 0.5;
        public static double LC1five = MyECULib.Properties.Settings.Default.LC1five; // 1.523;
        public static double LC1lambdaMultiplier = (LC1five - LC1zero) / 5.0;
        public static double AFRmultiplier = MyECULib.Properties.Settings.Default.AFRmultiplier; // 14.7;
        // fixed bridge resistor (temp sensors, 1K5)
        public static int rfix = 1500;
        //divider for reading 12V (47K and 10K)
        public static double vccmult = 57.0/10.0;

        public static double DegPerPulse = (720.0 / PulsesPer); // Degrees per pulse from wheel
        //public static int CylOffsPulses = (int)(CylOffs / DegPerPulse);
        //public static int TDC1 = (int)(TDCPulse + CylOffsPulses);

        public string ECUversionString = "MyECU";
        private StreamWriter sw;

        abstract public int Timeouts();
        abstract public bool Open();
        abstract public void DiscardBuffer();
        abstract public bool isOpen();
        abstract public string getData();

        // Steinhart-Hart Equation coefficients
        // These are known good for all Weber Marelli P7, P8, 1.6M and 15M ECU's temperature sensors
        public static double[] SHCoeffs = { 1.420572786078488e-003,
                                                  2.304603277353994e-004,
                                                  8.685490657201142e-007,
                                                  6.374542662257185e-008 };

        public virtual void Close()
        {
            if (sw != null)
            {
                sw.Close();
                sw = null;
            }
        }

        public bool setOutputFile(string filename)
        {
            try
            {
                sw = File.CreateText(filename);
                sw.NewLine = "\r\n";
                return true;
            }
            catch
            {
                return false;
            }
        }

        public MyECUData GetECUdata()
        {
            string data = getData().Trim();

            if (sw != null)
            {
                sw.WriteLine(data);
            }
            return new MyECUData(data);
        }
    }
}
